import {
  select,
  descending,
  utcParse,
  transition,
  scaleLinear,
  scalePoint,
  scaleOrdinal,
  schemeCategory10,
  extent,
  line,
  brushY,
} from 'd3';
import { scatter } from './scatterPlot';
import { bar } from './barChart';
import { lineChart } from './lineChart';
import { parallelCoordinates } from './parallelCoordinates';
import { data } from '@eschuman20/footballscorers';
import { observeResize } from '@curran/responsive-axes';

//Parallel Coordinates
data.forEach((d, i) => {
  d.id = i;
});

export const main = (container, { state, setState }) => {
  const dimensions = observeResize({
    state,
    setState,
    container,
  });

  if (dimensions === null) return;
  const { width, height } = dimensions;

  if (!state.xValue)
    setState((prevState) => ({
      ...prevState,
      xValue: 'Goals',
    }));

  if (!state.yValue)
    setState((prevState) => ({
      ...prevState,
      yValue: 'Shots',
    }));
  if (!state.yearValue)
    setState((prevState) => ({
      ...prevState,
      yearValue: 2016,
    }));

  if (!state.leagueValue)
    setState((prevState) => ({
      ...prevState,
      leagueValue: 'Serie A',
    }));

  // Scatter Plot
  const yearData = data.filter(
    (d) => d['Year'] === parseInt(state.yearValue),
  );
  const { hoveredValue } = state;
  const setHoveredValue = (d) => {
    setState((state) => ({
      ...state,
      hoveredValue: d,
    }));
  };

  // Bar Chart
  const nameColumn = 'Player Names';
  const n = 5;

  const leagueData = yearData.filter(
    (d) => d['League'] === state.leagueValue,
  );
  const sortedData = leagueData.sort((a, b) =>
    descending(a[state.yValue], b[state.yValue]),
  );
  const topNData = sortedData.slice(0, n);

  // Line Chart
  const counts = data.reduce((acc, d) => {
    const playerName = d['Player Names'];
    acc[playerName] = (acc[playerName] || 0) + 1;
    return acc;
  }, {});
  const filteredData1 = data.filter(
    (d) => counts[d['Player Names']] >= 3,
  );
  const filteredData = filteredData1.filter(
    (d) => d['League'] === state.leagueValue,
  );

  const parseYear = utcParse('%Y');
  // Parallel Coordinates
  if (!state.brushedIntervals) {
    setState((state) => ({
      ...state,
      brushedIntervals: {},
    }));
    return;
  }
  const { brushedIntervals } = state;

  const updateBrushedInterval = ({ column, interval }) => {
    setState((state) => ({
      ...state,
      brushedIntervals: {
        ...state.brushedIntervals,
        [column]: interval,
      },
    }));
  };

  const columns = [
    'Matches_Played',
    'Mins',
    'Shots',
    'OnTarget',
    'Goals',
  ];

  const columnTypes = {
    Matches_Played: 'quantitative',
    Mins: 'quantitative',
    Shots: 'quantitative',
    OnTarget: 'quantitative',
    Goals: 'quantitative',
    Substitution: 'quantitative',
    Shots_Per_Avg_Match: 'quantitative',
    On_Target_Per_Avg_Match: 'quantitative',
    xG: 'quantitative',
    xG_Per_Avg_Match: 'quantitative',
  };

  const colorValue = (d) => d.League;
  const idValue = (d) => d.id;
  const linelabel = (d) => d['Player Names'];

  ///////////////////////// Menus
  let ySelect = container.querySelector('select');
  let xSelect = container.querySelector('select');
  let yearSelect = container.querySelector('select');
  let leagueSelect = container.querySelector('select');

  if (!ySelect) {
    ySelect = document.createElement('select');
    ySelect.innerHTML = `
    <option value="Shots">Shots</option>
    <option value="Goals">Goals</option>
    <option value="Shots_Per_Avg_Match">Shots Per Match</option>
    <option value="OnTarget">Shots On Target</option>
    <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
    <option value="xG">xG Score</option>
    <option value="xG_Per_Avg_Match">xG Per Match</option>
    <option value="Matches_Played">Matches Played</option>
    <option value="Substitution">Substitutions</option>
    <option value="Mins">Minutes</option>
  `;

    ySelect.addEventListener('change', () => {
      setState((prevState) => ({
        ...state,
        xValue: xSelect.value,
        yValue: ySelect.value,
        leagueValue: leagueSelect.value,
        yearValue: yearSelect.value,
      }));
    });

    container.appendChild(ySelect);
  }

  if (!xSelect) {
    xSelect = document.createElement('select');
    xSelect.innerHTML = `
      <option value="Goals">Goals</option>
      <option value="Shots">Shots</option>
      <option value="Shots_Per_Avg_Match">Shots Per Match</option>
      <option value="OnTarget">Shots On Target</option>
      <option value="On_Target_Per_Avg_Match">Shots On Target Per Match</option>
      <option value="xG">xG Score</option>
      <option value="xG_Per_Avg_Match">xG Per Match</option>
      <option value="Matches_Played">Matches Played</option>
      <option value="Substitution">Substitutions</option>
      <option value="Mins">Minutes</option>
    `;

    xSelect.addEventListener('change', () => {
      setState((prevState) => ({
        ...state,
        xValue: xSelect.value,
        yValue: ySelect.value,
        leagueValue: leagueSelect.value,
        yearValue: yearSelect.value,
      }));
    });

    container.appendChild(xSelect);
  }

  if (!yearSelect) {
    yearSelect = document.createElement('select');
    yearSelect.innerHTML = `
    <option value=2016>2016</option>
    <option value=2017>2017</option>
    <option value=2018>2018</option>
    <option value=2019>2019</option>
    <option value=2020>2020</option>
  `;

    yearSelect.addEventListener('change', () => {
      setState((prevState) => ({
        ...state,
        xValue: xSelect.value,
        yValue: ySelect.value,
        leagueValue: leagueSelect.value,
        yearValue: yearSelect.value,
      }));
    });

    container.appendChild(yearSelect);
  }

  if (!leagueSelect) {
    leagueSelect = document.createElement('select');
    leagueSelect.innerHTML = `
    <option value="La Liga">La Liga</option>
    <option value="Serie A">Serie A</option>
    <option value="Premier League">Premier League</option>
    <option value="Bundesliga">Bundesliga</option>
    <option value="France Ligue 1">France Ligue 1</option>
    <option value="Primeira Liga">Primeira Liga</option>
    <option value="Campeonato Brasileiro SÃ©rie A">Campeonato Brasileiro SÃ©rie A</option>
    <option value="MLS">MLS</option>
    <option value="Eredivisie">Eredivisie</option>
  `;

    leagueSelect.addEventListener('change', () => {
      setState((prevState) => ({
        ...state,
        xValue: xSelect.value,
        yValue: ySelect.value,
        leagueValue: leagueSelect.value,
        yearValue: yearSelect.value,
      }));
    });

    container.appendChild(leagueSelect);
  }
  /////////////////////// SVG's

  const svg1Container = select(container)
    .selectAll('.svg1-container')
    .data([null])
    .join('div')
    .attr('class', 'svg1-container')
    .style('width', `${width / 2}px`)
    .style('height', `${height / 2}px`);

  const svg1 = svg1Container
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width / 2)
    .attr('height', height / 2);

  scatter(svg1, {
    data: yearData,
    xValue: (d) => d[state.xValue],
    xAxisLabelText: state.xValue,
    xAxisLabelOffset: 35,
    yValue: (d) => d[state.yValue],
    yAxisLabelText: state.yValue,
    yAxisLabelOffset: 17,
    innerRectFill: '#E8E8E8',
    circleRadius: 206 / 20,
    circleOpacity: 408 / 1000,
    marginTop: 40,
    marginBottom: 60,
    marginLeft: 54,
    marginRight: 20,
    width: width / 2,
    height: height / 2,
    colorLegendX: width / 2 - 110,
    colorLegendY: height / 2 - 220,
    colorLegendLabel: 'League',
    circleLabel: (d) => d['Player Names'],
    colorValue: (d) => d.League,
    setHoveredValue,
    hoveredValue,
  });

  const svg2Container = select(container)
    .selectAll('.svg2-container')
    .data([null])
    .join('div')
    .attr('class', 'svg2-container')
    .style('width', `${width / 2}px`)
    .style('height', `${height / 2}px`);

  const svg2 = svg2Container
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width / 2)
    .attr('height', height / 2);

  bar(svg2, {
    data: topNData,
    xValue: (d) => d[nameColumn],
    xAxisLabelText: 'Player Name',
    xAxisLabelOffset: 35,
    yValue: (d) => d[state.yValue],
    yAxisLabelText: state.yValue,
    yAxisLabelOffset: 27,
    innerRectFill: '#E8E8E8',
    padding: 12 / 100,
    marginTop: 20,
    marginBottom: 57,
    marginLeft: 74,
    marginRight: 20,
    width: width / 2,
    height: height / 2,
  });

  const svg3Container = select(container)
    .selectAll('.svg3-container')
    .data([null])
    .join('div')
    .attr('class', 'svg3-container')
    .style('position', 'absolute')
    .style('top', '0')
    .style('right', '0')
    .style('width', `${width / 2}px`)
    .style('height', `${height / 2}px`);

  const svg3 = svg3Container
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width / 2)
    .attr('height', height / 2);

  lineChart(svg3, {
    data: filteredData,
    xValue: (d) => {
      return parseYear(d.Year);
    },
    xAxisLabelText: 'Year',
    xAxisLabelOffset: 30,
    yValue: (d) => d[state.yValue],
    lineValue: (d) => d['Player Names'],
    yAxisLabelText: state.yValue,
    yAxisLabelOffset: 23,
    innerRectFill: '#E8E8E8',
    marginTop: 20,
    marginBottom: 50,
    marginLeft: 54,
    marginRight: 20,
    width: width / 2,
    height: height / 2,
    linelabel: (d) => d['Player Names'],
  });

  const svg4Container = select(container)
    .selectAll('.svg4-container')
    .data([null])
    .join('div')
    .attr('class', 'svg4-container')
    .style('position', 'absolute')
    .style('bottom', '0')
    .style('right', '0')
    .style('width', `${width / 2}px`)
    .style('height', `${height / 2}px`)
    .style('background', 'grey');

  const svg4 = svg4Container
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width / 2)
    .attr('height', height / 2);

  svg4.call(parallelCoordinates, {
    data: yearData,
    columns,
    columnTypes,
    colorValue,
    idValue,
    width: width / 2,
    height: height / 2,
    brushedIntervals,
    updateBrushedInterval,
    colorLegendX: width / 2 - 150,
    colorLegendY: height / 2 - 200,
    colorLegendLabel: 'League',
    linelabel,
  });