import {
  scaleLinear,
  scaleTime,
  scaleSequential,
  axisLeft,
  extent,
  line,
  flatGroup,
  interpolatePlasma,
  select,
} from 'd3';
import { axes } from './axes';

export const lineChart = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    yAxisLabelText,
    yAxisLabelOffset,
    lineValue,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
    lineOpacity,
    linelabel,
  },
) => {
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([marginLeft, width - marginRight]);
  console.log(extent(data, xValue));

  const yScale = scaleLinear(extent(data, yValue), [
    height - marginBottom,
    marginTop,
  ]);

  const colorScale = scaleSequential(
    extent(data, lineValue).reverse(),
    interpolatePlasma,
  );

  const innerWidth = width - marginLeft - marginRight;
  const innerHeight = height - marginTop - marginBottom;

  axes(svg, {
    width,
    height,
    xScale,
    xAxisLabelText,
    xAxisLabelOffset,
    yScale,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginBottom,
  });

  const lineGenerator = line(
    (d) => xScale(xValue(d)),
    (d) => yScale(yValue(d)),
  ).defined(yValue);

  const dataGrouped = flatGroup(data, lineValue);

  svg
    .selectAll('path.mark')
    .data(dataGrouped)
    .join('path')
    .attr('class', 'mark')
    .attr('fill', 'none')
    .attr('stroke-linecap', 'round')
    .attr('stroke', 'black')
    .attr('stroke-width', 3)
    .attr('d', (d) => lineGenerator(d[1]))
    .on('mouseover', function () {
      console.log('Mouseover event triggered');
      select(this).transition().attr('stroke-width', 10);

      select(this)
        .append('title')
        .text((d) => {
          console.log(d);
          return linelabel(d[1][0]);
        });
    })
    .on('mouseout', function () {
      select(this).transition().attr('stroke-width', 3);
      select(this).select('title').remove();
    });
};
